/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
/* 
 * Copyright (C) 2009,2010  Alexander Sack <asac@jwsdot.com>
 *
 * This file is part of:
 *     ntrack - Network Status Tracking for Desktop Applications
 *              http://launchpad.net/ntrack
 *
 *  ntrack is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as
 *  published by the Free Software Foundation, either version 3 of
 *  the License, or (at your option) any later version.
 *
 *  ntrack is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ntrack.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "config.h"

#include "ntrackbase.h"
#include "ntrackarch.h"
#include "ntrackarchapi.h"

#include <dirent.h>
#include <dlfcn.h>
#include <error.h>
#include <libgen.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>

#ifndef MODULES_DIR
#define MODULES_DIR "/usr/lib/ntrack/modules/"
#endif

static char buf[4][PATH_MAX] = {
	MODULES_DIR,
	"",
	"",
	""
};

void
ntrack_init (int *argc, char ***argv)
{
	char *dirn = dirname ((*argv)[0]);
	int i=0;
#ifdef DEVELOPER_BUILD
	sprintf (buf[i++], "%s/../../modules/.libs/", dirn);
	sprintf (buf[i++], "%s/../../../modules/.libs/", dirn);
#endif
	dirn = MODULES_DIR;
	sprintf (buf[i++], "%s", dirn);
	buf[i++][0] = 0;
}

struct _ntrack_module module;

ntrack_monitor_t*
ntrack_arch_new (ntrack_monitor_callback_f callback, ntrackpointer user_data)
{
	DIR *dir;
	struct dirent *dirent;
	void *dlhandle;
	const char *dirpath;
	static int ftbl_initialized = 0;
	ntrack_monitor_t *self = 0;
	int i;

	if (ftbl_initialized)
		return (module.ftbl->new) (callback, user_data);

	for (i=0; !self && buf[i] && buf[i][0]; i++) {
		dir = opendir (buf[i]);
		dirpath = buf[i];
		
		while (!self && dir && (dirent = readdir (dir))) {
			char *filepath = malloc (sizeof(char) *
			                         (strlen(dirent->d_name) + 4096 + 2));
			sprintf (filepath, "%s%s", dirpath, dirent->d_name);
			dlhandle = dlopen (filepath, RTLD_LAZY | RTLD_LOCAL);
			if (!dlhandle) {
				continue;
			}
			free(filepath);

			module = *( (struct _ntrack_module*) dlsym (dlhandle, "ntrack_module"));

			if (module.ftbl)
			{
				self = (module.ftbl->new) (callback, user_data);
				if (self) {
					printf ("ntrack backend selected and created: %s\n", dirent->d_name);
					ftbl_initialized = 1;
				}
			} else {
				dlclose (dlhandle);
			}
		}
		if (dir)
			closedir (dir);
	}

	if (!self)
		printf ("no backend found finale.\n");
	return self;
}

int*
ntrack_arch_get_rfds (ntrack_monitor_t *self)
{
	if (module.ftbl->get_rfds)
		return (module.ftbl->get_rfds) (self);
	else
		printf ("no rfds func found, but called\n");

	return 0;
}

int
ntrack_arch_process_data (ntrackpointer self, int* fds)
{
	if (module.ftbl->process_data)
		return (module.ftbl->process_data) (self, fds);
	else
		printf ("no process_data_func found, but called\n");

	return 0;
}

void
ntrack_arch_free (ntrackpointer self)
{
	if (module.ftbl->free)
		(module.ftbl->free) (self);
	else
		printf ("no process_data_func found, but called\n");
	return;
}

